@@ -10,12 +10,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import itertools
 import json
 import time
 import uuid
 
 from tempest.lib import exceptions
 
+from openstackclient.compute.v2 import server as v2_server
 from openstackclient.tests.functional.compute.v2 import common
 from openstackclient.tests.functional.volume.v2 import common as volume_common
 
@@ -63,6 +65,49 @@ class ServerTests(common.ComputeTestCase):
         self.assertNotIn(name1, col_name)
         self.assertIn(name2, col_name)
 
+    def test_server_list_with_marker_and_deleted(self):
+        """Test server list with deleted and marker"""
+        cmd_output = self.server_create(cleanup=False)
+        name1 = cmd_output['name']
+        cmd_output = self.server_create(cleanup=False)
+        name2 = cmd_output['name']
+        id2 = cmd_output['id']
+        self.wait_for_status(name1, "ACTIVE")
+        self.wait_for_status(name2, "ACTIVE")
+
+        # Test list --marker with ID
+        cmd_output = json.loads(self.openstack(
+            'server list -f json --marker ' + id2
+        ))
+        col_name = [x["Name"] for x in cmd_output]
+        self.assertIn(name1, col_name)
+
+        # Test list --marker with Name
+        cmd_output = json.loads(self.openstack(
+            'server list -f json --marker ' + name2
+        ))
+        col_name = [x["Name"] for x in cmd_output]
+        self.assertIn(name1, col_name)
+
+        self.openstack('server delete --wait ' + name1)
+        self.openstack('server delete --wait ' + name2)
+
+        # Test list --deleted --marker with ID
+        cmd_output = json.loads(self.openstack(
+            'server list -f json --deleted --marker ' + id2
+        ))
+        col_name = [x["Name"] for x in cmd_output]
+        self.assertIn(name1, col_name)
+
+        # Test list --deleted --marker with Name
+        try:
+            cmd_output = json.loads(self.openstack(
+                'server list -f json --deleted --marker ' + name2
+            ))
+        except exceptions.CommandFailed as e:
+            self.assertIn('marker [%s] not found (HTTP 400)' % (name2),
+                          e.stderr.decode('utf-8'))
+
     def test_server_list_with_changes_before(self):
         """Test server list.
 
@@ -187,7 +232,7 @@ class ServerTests(common.ComputeTestCase):
         ))
         # Really, shouldn't this be a list?
         self.assertEqual(
-            "a='b', c='d'",
+            {'a': 'b', 'c': 'd'},
             cmd_output['properties'],
         )
 
@@ -201,7 +246,7 @@ class ServerTests(common.ComputeTestCase):
             name
         ))
         self.assertEqual(
-            "c='d'",
+            {'c': 'd'},
             cmd_output['properties'],
         )
 
@@ -302,6 +347,14 @@ class ServerTests(common.ComputeTestCase):
             #                DevStack without cells.
             self.skipTest("No Network service present")
 
+        def _chain_addresses(addresses):
+            # Flatten a dict of lists mapping network names to IP addresses,
+            # returning only the IP addresses
+            #
+            # >>> _chain_addresses({'private': ['10.1.0.32', '172.24.5.41']})
+            # ['10.1.0.32', '172.24.5.41']
+            return itertools.chain(*[*addresses.values()])
+
         cmd_output = self.server_create()
         name = cmd_output['name']
         self.wait_for_status(name, "ACTIVE")
@@ -343,7 +396,7 @@ class ServerTests(common.ComputeTestCase):
                 'server show -f json ' +
                 name
             ))
-            if floating_ip not in cmd_output['addresses']:
+            if floating_ip not in _chain_addresses(cmd_output['addresses']):
                 # Hang out for a bit and try again
                 print('retrying floating IP check')
                 wait_time += 10
@@ -353,7 +406,7 @@ class ServerTests(common.ComputeTestCase):
 
         self.assertIn(
             floating_ip,
-            cmd_output['addresses'],
+            _chain_addresses(cmd_output['addresses']),
         )
 
         # detach ip
@@ -373,7 +426,7 @@ class ServerTests(common.ComputeTestCase):
                 'server show -f json ' +
                 name
             ))
-            if floating_ip in cmd_output['addresses']:
+            if floating_ip in _chain_addresses(cmd_output['addresses']):
                 # Hang out for a bit and try again
                 print('retrying floating IP check')
                 wait_time += 10
@@ -387,7 +440,7 @@ class ServerTests(common.ComputeTestCase):
         ))
         self.assertNotIn(
             floating_ip,
-            cmd_output['addresses'],
+            _chain_addresses(cmd_output['addresses']),
         )
 
     def test_server_reboot(self):
@@ -466,6 +519,20 @@ class ServerTests(common.ComputeTestCase):
             server['name'],
         )
 
+        # check that image indicates server "booted from volume"
+        self.assertEqual(
+            v2_server.IMAGE_STRING_FOR_BFV,
+            server['image'],
+        )
+        # check server list too
+        servers = json.loads(self.openstack(
+            'server list -f json'
+        ))
+        self.assertEqual(
+            v2_server.IMAGE_STRING_FOR_BFV,
+            servers[0]['Image']
+        )
+
         # check volumes
         cmd_output = json.loads(self.openstack(
             'volume show -f json ' +
@@ -507,7 +574,90 @@ class ServerTests(common.ComputeTestCase):
             cmd_output['status'],
         )
 
-    def test_server_boot_with_bdm_snapshot(self):
+    def _test_server_boot_with_bdm_volume(self, use_legacy):
+        """Test server create from volume, server delete"""
+        # get volume status wait function
+        volume_wait_for = volume_common.BaseVolumeTests.wait_for_status
+
+        # create source empty volume
+        volume_name = uuid.uuid4().hex
+        cmd_output = json.loads(self.openstack(
+            'volume create -f json ' +
+            '--size 1 ' +
+            volume_name
+        ))
+        volume_id = cmd_output["id"]
+        self.assertIsNotNone(volume_id)
+        self.addCleanup(self.openstack, 'volume delete ' + volume_name)
+        self.assertEqual(volume_name, cmd_output['name'])
+        volume_wait_for("volume", volume_name, "available")
+
+        if use_legacy:
+            bdm_arg = f'--block-device-mapping vdb={volume_name}'
+        else:
+            bdm_arg = (
+                f'--block-device '
+                f'device_name=vdb,source_type=volume,boot_index=1,'
+                f'uuid={volume_id}'
+            )
+
+        # create server
+        server_name = uuid.uuid4().hex
+        server = json.loads(self.openstack(
+            'server create -f json ' +
+            '--flavor ' + self.flavor_name + ' ' +
+            '--image ' + self.image_name + ' ' +
+            bdm_arg + ' ' +
+            self.network_arg + ' ' +
+            '--wait ' +
+            server_name
+        ))
+        self.assertIsNotNone(server["id"])
+        self.addCleanup(self.openstack, 'server delete --wait ' + server_name)
+        self.assertEqual(
+            server_name,
+            server['name'],
+        )
+
+        # check server volumes_attached, format is
+        # {"volumes_attached": "id='2518bc76-bf0b-476e-ad6b-571973745bb5'",}
+        cmd_output = json.loads(self.openstack(
+            'server show -f json ' +
+            server_name
+        ))
+        volumes_attached = cmd_output['volumes_attached']
+        self.assertIsNotNone(volumes_attached)
+
+        # check volumes
+        cmd_output = json.loads(self.openstack(
+            'volume show -f json ' +
+            volume_name
+        ))
+        attachments = cmd_output['attachments']
+        self.assertEqual(
+            1,
+            len(attachments),
+        )
+        self.assertEqual(
+            server['id'],
+            attachments[0]['server_id'],
+        )
+        self.assertEqual(
+            "in-use",
+            cmd_output['status'],
+        )
+
+    def test_server_boot_with_bdm_volume(self):
+        """Test server create from image with bdm volume, server delete"""
+        self._test_server_boot_with_bdm_volume(use_legacy=False)
+
+    # TODO(stephenfin): Remove when we drop support for the
+    # '--block-device-mapping' option
+    def test_server_boot_with_bdm_volume_legacy(self):
+        """Test server create from image with bdm volume, server delete"""
+        self._test_server_boot_with_bdm_volume(use_legacy=True)
+
+    def _test_server_boot_with_bdm_snapshot(self, use_legacy):
         """Test server create from image with bdm snapshot, server delete"""
         # get volume status wait function
         volume_wait_for = volume_common.BaseVolumeTests.wait_for_status
@@ -521,12 +671,8 @@ class ServerTests(common.ComputeTestCase):
             empty_volume_name
         ))
         self.assertIsNotNone(cmd_output["id"])
-        self.addCleanup(self.openstack,
-                        'volume delete ' + empty_volume_name)
-        self.assertEqual(
-            empty_volume_name,
-            cmd_output['name'],
-        )
+        self.addCleanup(self.openstack, 'volume delete ' + empty_volume_name)
+        self.assertEqual(empty_volume_name, cmd_output['name'])
         volume_wait_for("volume", empty_volume_name, "available")
 
         # create snapshot of source empty volume
@@ -536,7 +682,8 @@ class ServerTests(common.ComputeTestCase):
             '--volume ' + empty_volume_name + ' ' +
             empty_snapshot_name
         ))
-        self.assertIsNotNone(cmd_output["id"])
+        empty_snapshot_id = cmd_output["id"]
+        self.assertIsNotNone(empty_snapshot_id)
         # Deleting volume snapshot take time, so we need to wait until the
         # snapshot goes. Entries registered by self.addCleanup will be called
         # in the reverse order, so we need to register wait_for_delete first.
@@ -550,14 +697,26 @@ class ServerTests(common.ComputeTestCase):
         )
         volume_wait_for("volume snapshot", empty_snapshot_name, "available")
 
+        if use_legacy:
+            bdm_arg = (
+                f'--block-device-mapping '
+                f'vdb={empty_snapshot_name}:snapshot:1:true'
+            )
+        else:
+            bdm_arg = (
+                f'--block-device '
+                f'device_name=vdb,uuid={empty_snapshot_id},'
+                f'source_type=snapshot,volume_size=1,'
+                f'delete_on_termination=true,boot_index=1'
+            )
+
         # create server with bdm snapshot
         server_name = uuid.uuid4().hex
         server = json.loads(self.openstack(
             'server create -f json ' +
             '--flavor ' + self.flavor_name + ' ' +
             '--image ' + self.image_name + ' ' +
-            '--block-device-mapping '
-            'vdb=' + empty_snapshot_name + ':snapshot:1:true ' +
+            bdm_arg + ' ' +
             self.network_arg + ' ' +
             '--wait ' +
             server_name
@@ -576,8 +735,8 @@ class ServerTests(common.ComputeTestCase):
             server_name
         ))
         volumes_attached = cmd_output['volumes_attached']
-        self.assertTrue(volumes_attached.startswith('id='))
-        attached_volume_id = volumes_attached.replace('id=', '')
+        self.assertIsNotNone(volumes_attached)
+        attached_volume_id = volumes_attached[0]["id"]
 
         # check the volume that attached on server
         cmd_output = json.loads(self.openstack(
@@ -614,7 +773,17 @@ class ServerTests(common.ComputeTestCase):
             # the attached volume had been deleted
             pass
 
-    def test_server_boot_with_bdm_image(self):
+    def test_server_boot_with_bdm_snapshot(self):
+        """Test server create from image with bdm snapshot, server delete"""
+        self._test_server_boot_with_bdm_snapshot(use_legacy=False)
+
+    # TODO(stephenfin): Remove when we drop support for the
+    # '--block-device-mapping' option
+    def test_server_boot_with_bdm_snapshot_legacy(self):
+        """Test server create from image with bdm snapshot, server delete"""
+        self._test_server_boot_with_bdm_snapshot(use_legacy=True)
+
+    def _test_server_boot_with_bdm_image(self, use_legacy):
         # Tests creating a server where the root disk is backed by the given
         # --image but a --block-device-mapping with type=image is provided so
         # that the compute service creates a volume from that image and
@@ -622,6 +791,32 @@ class ServerTests(common.ComputeTestCase):
         # marked as delete_on_termination=True so it will be automatically
         # deleted when the server is deleted.
 
+        if use_legacy:
+            # This means create a 1GB volume from the specified image, attach
+            # it to the server at /dev/vdb and delete the volume when the
+            # server is deleted.
+            bdm_arg = (
+                f'--block-device-mapping '
+                f'vdb={self.image_name}:image:1:true '
+            )
+        else:
+            # get image ID
+            cmd_output = json.loads(self.openstack(
+                'image show -f json ' +
+                self.image_name
+            ))
+            image_id = cmd_output['id']
+
+            # This means create a 1GB volume from the specified image, attach
+            # it to the server at /dev/vdb and delete the volume when the
+            # server is deleted.
+            bdm_arg = (
+                f'--block-device '
+                f'device_name=vdb,uuid={image_id},'
+                f'source_type=image,volume_size=1,'
+                f'delete_on_termination=true,boot_index=1'
+            )
+
         # create server with bdm type=image
         # NOTE(mriedem): This test is a bit unrealistic in that specifying the
         # same image in the block device as the --image option does not really
@@ -633,11 +828,7 @@ class ServerTests(common.ComputeTestCase):
             'server create -f json ' +
             '--flavor ' + self.flavor_name + ' ' +
             '--image ' + self.image_name + ' ' +
-            '--block-device-mapping '
-            # This means create a 1GB volume from the specified image, attach
-            # it to the server at /dev/vdb and delete the volume when the
-            # server is deleted.
-            'vdb=' + self.image_name + ':image:1:true ' +
+            bdm_arg + ' ' +
             self.network_arg + ' ' +
             '--wait ' +
             server_name
@@ -656,8 +847,8 @@ class ServerTests(common.ComputeTestCase):
             server_name
         ))
         volumes_attached = cmd_output['volumes_attached']
-        self.assertTrue(volumes_attached.startswith('id='))
-        attached_volume_id = volumes_attached.replace('id=', '')
+        self.assertIsNotNone(volumes_attached)
+        attached_volume_id = volumes_attached[0]["id"]
 
         # check the volume that attached on server
         cmd_output = json.loads(self.openstack(
@@ -701,6 +892,14 @@ class ServerTests(common.ComputeTestCase):
             # the attached volume had been deleted
             pass
 
+    def test_server_boot_with_bdm_image(self):
+        self._test_server_boot_with_bdm_image(use_legacy=False)
+
+    # TODO(stephenfin): Remove when we drop support for the
+    # '--block-device-mapping' option
+    def test_server_boot_with_bdm_image_legacy(self):
+        self._test_server_boot_with_bdm_image(use_legacy=True)
+
     def test_boot_from_volume(self):
         # Tests creating a server using --image and --boot-from-volume where
         # the compute service will create a root volume of the specified size
@@ -730,19 +929,21 @@ class ServerTests(common.ComputeTestCase):
             server_name
         ))
         volumes_attached = cmd_output['volumes_attached']
-        self.assertTrue(volumes_attached.startswith('id='))
-        attached_volume_id = volumes_attached.replace('id=', '')
-        # Don't leak the volume when the test exits.
-        self.addCleanup(self.openstack, 'volume delete ' + attached_volume_id)
+        self.assertIsNotNone(volumes_attached)
+        attached_volume_id = volumes_attached[0]["id"]
+        for vol in volumes_attached:
+            self.assertIsNotNone(vol['id'])
+            # Don't leak the volume when the test exits.
+            self.addCleanup(self.openstack, 'volume delete ' + vol['id'])
 
         # Since the server is volume-backed the GET /servers/{server_id}
-        # response will have image=''.
-        self.assertEqual('', cmd_output['image'])
+        # response will have image='N/A (booted from volume)'.
+        self.assertEqual(v2_server.IMAGE_STRING_FOR_BFV, cmd_output['image'])
 
         # check the volume that attached on server
         cmd_output = json.loads(self.openstack(
             'volume show -f json ' +
-            attached_volume_id
+            volumes_attached[0]["id"]
         ))
         # The volume size should be what we specified on the command line.
         self.assertEqual(1, int(cmd_output['size']))
@@ -796,8 +997,7 @@ class ServerTests(common.ComputeTestCase):
         server = json.loads(self.openstack(
             'server show -f json ' + server_name
         ))
-        self.assertIsNotNone(server['addresses'])
-        self.assertEqual('', server['addresses'])
+        self.assertEqual({}, server['addresses'])
 
     def test_server_create_with_security_group(self):
         """Test server create with security group ID and name"""
@@ -836,14 +1036,21 @@ class ServerTests(common.ComputeTestCase):
 
         self.assertIsNotNone(server['id'])
         self.assertEqual(server_name, server['name'])
-        self.assertIn(str(security_group1['id']), server['security_groups'])
-        self.assertIn(str(security_group2['id']), server['security_groups'])
+        sec_grp = ""
+        for sec in server['security_groups']:
+            sec_grp += sec['name']
+        self.assertIn(str(security_group1['id']), sec_grp)
+        self.assertIn(str(security_group2['id']), sec_grp)
         self.wait_for_status(server_name, 'ACTIVE')
         server = json.loads(self.openstack(
             'server show -f json ' + server_name
         ))
-        self.assertIn(sg_name1, server['security_groups'])
-        self.assertIn(sg_name2, server['security_groups'])
+        # check if security group exists in list
+        sec_grp = ""
+        for sec in server['security_groups']:
+            sec_grp += sec['name']
+        self.assertIn(sg_name1, sec_grp)
+        self.assertIn(sg_name2, sec_grp)
 
     def test_server_create_with_empty_network_option_latest(self):
         """Test server create with empty network option in nova 2.latest."""
